"""
Just shortens some lines to make code look better.
"""

FONT_ATTRIBUTES = ['font_name', 'font_size', 'font_bold', 'font_bold']
ANY_QUESTIONS = 'In case you have any questions, please ask them at https://github.com/Rednax-Code/pygame_ui/discussions'
ISSUE_REPORT = '[Note] if the above statement seems incorrect, please report this issue at https://github.com/Rednax-Code/pygame_ui/issues. Thank you :)\n'
MUST_BE = 'Must be one of the following:\n'
MORE_INFO = 'Check the documentation for more information about '
ALPHABET = ['a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z']
ANCHORS = {'top': -1, 'bottom': 1, 'left': -1, 'right': 1, 'center': 0}